function isEven(n) {
  return n % 2 === 0;
}

function toFixed(n) {
  return Number.parseFloat(n).toFixed(2);
}

function radToDeg(radians) {
  return radians * 180 / Math.PI;
}

function edgePoint(x1, y1, x2, y2, maxX, maxY) {
  let pointCoords,
      xDiff = x2 - x1,
      yDiff = y2 - y1,
      xIntercept = y => (y - y1) * xDiff / yDiff + x1,
      yIntercept = x => (x - x1) * yDiff / xDiff + y1;

  if (xDiff > 0 && yDiff > 0) {
    let yWhenXisMax = yIntercept(maxX);
    let xWhenYisMax = xIntercept(maxY);

    if (xWhenYisMax <= maxX) {
      pointCoords = [xWhenYisMax, maxY];
    } else {
      pointCoords = [maxX, yWhenXisMax];
    }
  } else if (xDiff > 0 && yDiff < 0) {
    let yWhenXisMax = yIntercept(maxX);
    let xWhenYisZero = xIntercept(0);

    if (xWhenYisZero <= maxX) {
      pointCoords = [xWhenYisZero, 0];
    } else {
      pointCoords = [maxX, yWhenXisMax];
    }
  } else if (xDiff < 0 && yDiff < 0) {
    let yWhenXisZero = yIntercept(0);
    let xWhenYisZero = xIntercept(0);

    if (yWhenXisZero >= 0) {
      pointCoords = [0, yWhenXisZero];
    } else {
      pointCoords = [xWhenYisZero, 0];
    }
  } else {
    let yWhenXisZero = yIntercept(0);
    let xWhenYisMax = xIntercept(maxY);

    if (yWhenXisZero <= maxY) {
      pointCoords = [0, yWhenXisZero];
    } else {
      pointCoords = [xWhenYisMax, maxY];
    }
  }

  return pointCoords;
}

let getPointCoords = (x, y) => {
  let point = document.querySelector(`[data-x="${x}"][data-y="${y}"]`);

  return [point.x.baseVal.value, point.y.baseVal.value]
};

let svgns = "http://www.w3.org/2000/svg",
    svg = document.querySelector('svg'),
    rect = document.querySelector('rect#map');

const COLUMN_COUNT = 33,
      ROW_COUNT = 25,
      HORZ_POINT_DISTANCE = 1.005,
      VERT_POINT_DISTANCE = Math.sqrt(3) * HORZ_POINT_DISTANCE / 2,
      ALTERNATING_OFFSET = HORZ_POINT_DISTANCE / 2,
      [X_OFFSET, Y_OFFSET] = [0.25, 0.45],
      [COLUMNS, ROWS] = [COLUMN_COUNT, ROW_COUNT].map(n => [...Array(n).keys()]),
      POINTS = ROWS.map(y => COLUMNS.map(x => [x, y]));

const FIRING_ARC_SIZE = {
        'small': Math.atan(HORZ_POINT_DISTANCE / (6 * VERT_POINT_DISTANCE)),
        'medium': Math.atan((HORZ_POINT_DISTANCE / 2) / VERT_POINT_DISTANCE),
        'large': Math.atan((21 * HORZ_POINT_DISTANCE) / (6 * VERT_POINT_DISTANCE))
      }

POINTS.forEach((row, index) => row.forEach(([x, y]) => {
  var cx = x * HORZ_POINT_DISTANCE + X_OFFSET + (isEven(index) ? ALTERNATING_OFFSET : 0),
      cy = y * HORZ_POINT_DISTANCE * VERT_POINT_DISTANCE + Y_OFFSET,
      point = document.createElementNS(svgns, 'use');

  point.setAttributeNS(null, 'href', `#point`);
  point.setAttributeNS(null, 'x', `${cx}in`);
  point.setAttributeNS(null, 'y', `${cy}in`);
  point.dataset.x = x;
  point.dataset.y = y;

  point.addEventListener('click', e => {
    let selectedSoldier = document.querySelector('.soldier-record.selected');

    if (selectedSoldier) {
      let counter = document.createElementNS(svgns, 'circle');
      let text = document.createElementNS(svgns, 'text');

      counter.setAttributeNS(null, 'cx', `${cx}in`);
      counter.setAttributeNS(null, 'cy', `${cy}in`);
      counter.setAttributeNS(null, 'r', '0.25in');
      counter.dataset.troopNumber = selectedSoldier.dataset.troopNumber;
      counter.dataset.troopAllegiance = selectedSoldier.dataset.troopAllegiance;
      counter.classList.add('counter');

      text.setAttributeNS(null, 'text-anchor', 'middle');
      text.setAttributeNS(null, 'x', `${cx}in`);
      text.setAttributeNS(null, 'y', `${cy + 0.25}in`);
      text.dataset.troopNumber = selectedSoldier.dataset.troopNumber;
      text.dataset.troopAllegiance = selectedSoldier.dataset.troopAllegiance;
      text.textContent = `${selectedSoldier.dataset.troopNumber}`;
      text.classList.add('counter');

      document.querySelectorAll(
        `.counter[data-troop-number="${selectedSoldier.dataset.troopNumber}"][data-troop-allegiance="${selectedSoldier.dataset.troopAllegiance}"]`
      ).forEach(el => el.remove());

      svg.appendChild(counter);
      svg.appendChild(text);
    }
  });

  svg.appendChild(point);
}));

rect.addEventListener('mousemove', e => {
  var rect = e.target.getBoundingClientRect();
  var x = e.clientX - rect.left;
  var y = e.clientY - rect.top;
  let [maxX, maxY] = [e.target.width, e.target.height].map(v => v.baseVal.valueInSpecifiedUnits);
  let [maxXpx, maxYpx] = [e.target.width, e.target.height].map(v => v.baseVal.value);

  // console.log('x', `${toFixed(x / rect.width * maxX)}"`, 'y', `${toFixed(y / rect.height * maxY)}"`);

  let aim = document.querySelector('line.firing-arc.active');
  // let p = document.querySelector('polyline.firing-arc.active');
  let p = document.querySelector('polygon.firing-arc.active');

  // TODO: handle horizontal and vertical lines

  if (aim && p) {
    let [x1, y1] = [aim.x1, aim.y1].map(v => v.baseVal.valueInSpecifiedUnits);
    let [x2, y2] = [x / rect.width * maxX, y / rect.height * maxY];

    let {x: x1px, y: y1px} = p.points[1];
    let [x2px, y2px] = [x / rect.width * maxXpx, y / rect.height * maxYpx];

    let xDiff = x2px - x1px;
    let yDiff = -(y2px - y1px);
    let angle = Math.abs(Math.atan(yDiff / xDiff));

    if (xDiff < 0 && yDiff > 0) {
      angle = Math.PI - angle;
    } else if (xDiff < 0 && yDiff < 0) {
      angle = Math.PI + angle;
    } else if (xDiff > 0 && yDiff < 0) {
      angle = 2 * Math.PI - angle;
    }

    console.log('angle:', `${toFixed(radToDeg(angle))}\u00B0`, `${angle}rad`);

    let arcAngle = FIRING_ARC_SIZE[p.dataset.size];
    let distance = Math.sqrt((x2px - x1px)**2 + (y2px - y1px)**2);
    let yDelta = distance * Math.cos(angle) * Math.tan(arcAngle);
    let xDelta = distance * Math.sin(angle) * Math.tan(arcAngle);

    let [newY1, newX1] = [y2px + yDelta, x2px + xDelta];
    let [newY2, newX2] = [y2px - yDelta, x2px - xDelta];

    [newX1, newY1] = edgePoint(x1px, y1px, newX1, newY1, maxXpx, maxYpx);
    [newX2, newY2] = edgePoint(x1px, y1px, newX2, newY2, maxXpx, maxYpx);

    // var sameEdgeConditions = [

    // ]

    // if (sameEdgeConditions)

    // p.setAttributeNS(null, 'points', `${newX},${newY} ${x1px},${y1px} ${newX},${newY}`);
    // p.setAttributeNS(null, 'points', `${x2px},${y2px} ${x1px},${y1px} ${x2px},${y2px}`);
    p.setAttributeNS(null, 'points', `${newX1},${newY1} ${x1px},${y1px} ${newX2},${newY2}`);

    let [newX, newY] = edgePoint(x1, y1, x2, y2, maxX, maxY);

    aim.setAttributeNS(null, 'x2', `${newX}in`);
    aim.setAttributeNS(null, 'y2', `${newY}in`);
  }
});

document.querySelectorAll('.soldier-record').forEach(el =>
  el.addEventListener('click', e => {
    if (e.target.classList.contains('selected')) {
      e.target.classList.remove('selected');
    } else {
      document.querySelectorAll('.soldier-record.selected').forEach(el =>
        el.classList.remove('selected')
      );
      e.target.classList.add('selected');
    }
  })
);

document.querySelectorAll('.set-firing-arc').forEach(el => el.addEventListener('click', e => {
  let selectedSoldier = document.querySelector('.soldier-record.selected');

  if (selectedSoldier) {
    let {troopNumber, troopAllegiance} = selectedSoldier.dataset;

    let existingArcs = document.querySelectorAll(
      `.firing-arc[data-troop-number="${troopNumber}"][data-troop-allegiance="${troopAllegiance}"]`
    );

    if (existingArcs.length > 0) {
      existingArcs.forEach(el => el.remove());
    }

    let counter = document.querySelector(
      `circle.counter[data-troop-number="${troopNumber}"][data-troop-allegiance="${troopAllegiance}"]`
    );

    let arcLayer = document.getElementById('firing-arcs');

    let aim = document.createElementNS(svgns, 'line');

    aim.classList.add('firing-arc', 'active');
    aim.dataset.troopNumber = troopNumber;
    aim.dataset.troopAllegiance = troopAllegiance;
    aim.dataset.size = e.target.dataset.size;
    aim.setAttributeNS(null, 'x1', counter.cx.baseVal.valueAsString);
    aim.setAttributeNS(null, 'y1', counter.cy.baseVal.valueAsString);
    aim.setAttributeNS(null, 'x2', counter.cx.baseVal.valueAsString);
    aim.setAttributeNS(null, 'y2', counter.cy.baseVal.valueAsString);
    // aim.style.transformOrigin = `${counter.cx.baseVal.valueAsString} ${counter.cy.baseVal.valueAsString}`;

    // svg.appendChild(aim);
    arcLayer.prepend(aim);

    aim.addEventListener('click', e => {
      // e.target.classList.remove('active');
      document.querySelectorAll('.firing-arc.active').forEach(el => el.classList.remove('active'));
      document.querySelector('circle#point').style.display = '';
    });

    let pivotPoint = [counter.cx.baseVal.value, counter.cy.baseVal.value];
    // let firingArc = document.createElementNS(svgns, 'polyline');
    let firingArc = document.createElementNS(svgns, 'polygon');
    firingArc.classList.add('firing-arc', 'active');
    firingArc.dataset.troopNumber = troopNumber;
    firingArc.dataset.troopAllegiance = troopAllegiance;
    firingArc.dataset.size = e.target.dataset.size;
    firingArc.setAttributeNS(null, 'points', `${pivotPoint} ${pivotPoint} ${pivotPoint}`);

    arcLayer.prepend(firingArc);
    // svg.appendChild(firingArc);

    document.querySelector('circle#point').style.display = 'none';
  }
}));

let smlArc = document.createElementNS(svgns, 'polygon');
smlArc.setAttributeNS(null, 'id', 'sml-arc');
let smlArcPoints = `${getPointCoords(1, 24)} ${getPointCoords(1, 21)} ${getPointCoords(2, 21)}`;
smlArc.setAttributeNS(null, 'points', smlArcPoints);

svg.appendChild(smlArc);

let medArc = document.createElementNS(svgns, 'polygon');
medArc.setAttributeNS(null, 'id', 'med-arc');
let medArcPoints = `${getPointCoords(4, 24)} ${getPointCoords(3, 21)} ${getPointCoords(6, 21)}`;
medArc.setAttributeNS(null, 'points', medArcPoints);

svg.appendChild(medArc);

let lrgArc = document.createElementNS(svgns, 'polygon');
lrgArc.setAttributeNS(null, 'id', 'lrg-arc');
let lrgArcPoints = `${getPointCoords(19, 24)} ${getPointCoords(9, 21)} ${getPointCoords(30, 21)}`;
lrgArc.setAttributeNS(null, 'points', lrgArcPoints);

svg.appendChild(lrgArc);

// arc = document.querySelector('#med-arc');
// let [ox, oy] = getPointCoords(4, 24);
// arc.style.transformOrigin = `${ox}px ${oy}px`;
// arc.style.transform = 'rotate(90deg)';

var smlArcAngle = radToDeg((Math.atan(HORZ_POINT_DISTANCE / (6 * VERT_POINT_DISTANCE)) * 2));
var medArcAngle = radToDeg((Math.atan((3 * HORZ_POINT_DISTANCE) / (6 * VERT_POINT_DISTANCE)) * 2));
var lrgArcAngle = radToDeg((Math.atan((21 * HORZ_POINT_DISTANCE) / (6 * VERT_POINT_DISTANCE)) * 2));